%Creating Grid System for Seabed Topogrophy
%Note: x, y and z tables for grids should contain same amount of data
%Also, names of imported tables should be as following:
%x table >>> xyzmapz, y table >>> xyzmapy, z table >>> xyzmapz
x = xyzmapx{:,:};
y = xyzmapy{:,:};
z = xyzmapz{:,:};

%Plotting seabed map
figure(1)
h=surf(x,y,z)
set(h,'LineStyle','none')
zlim([-180 100])    %This figure settings have been made for the used data 
xlim([-35000 35500]) %for other data these can be changed 
ylim([-35000 35500])
xlabel('x');
ylabel('y');
zlabel('z');
pbaspect([1 1 1]);
c = colorbar;
c.Label.String = 'Sea Depth (m)';
grid off

%Creating nodes for the Graph Object
[m,n]=size(x);
n_values=numel(z);
Nodes=zeros(n_values,3);
xt=transpose(x);
yt=transpose(y);
zt=transpose(z);
Nodes(:,1)=xt(:);
Nodes(:,2)=yt(:);
Nodes(:,3)=zt(:);
clear xt yt zt 

% Calculating Euclidean distances between each nodes
dist = pdist(Nodes); %This is available MATLAB function to calculate 
                     %Euclidean ditances

% Convert determined data to adjacency matrix
adj_m = squareform(dist);
clear dist
adj_m=forcetoneighbour(adj_m,m,n); %This is wriiten function for removing 
                         %edges between the nodes which are not neighbours

% Generating Graph object
names = cellstr(string(1:n_values));
R = graph(adj_m);
R.Nodes = array2table(Nodes,'VariableNames',{'X','Y','Z'});
R.Nodes.Name = names';



