%Input data:
%Adding co-ordinate information for riserbase and templates
RBC = [0 0];
T1= [-21000 -20500]; T2= [-28167 166.67]; T3= [-21333 21833];
T4= [20000 21333]; T5= [28833 -333.33]; T6= [22667 -21833];

%If additional templates have been added, they should also be added to the 
%below cell array
T={T1 T2 T3 T4 T5 T6};
Tn=numel(T);

%Adding minimum allowable radius of curvature
%Also, dx and dy values for the created grid system 
Rmin=10000; %Adding minimum radius of curvature
dx=0.469798658*1000; %dx of one grid in meter
dy=0.463576159*1000; %dy of one grid in meter

%End of Input data

%Determining node number of given co-ordinates
x_RBC= round(abs(x(1,1)-RBC(1,1))/dx+1);
y_RBC= round(abs(y(1,1)-RBC(1,2))/dy+1);
n_RBC= round((y_RBC-1)*n+x_RBC);
x_T=cell(1,Tn);
y_T=cell(1,Tn);
n_T=cell(1,Tn);
for i=1:Tn
x_T{i}= round(abs(x(1,1)-T{i}(1,1))/dx+1);
y_T{i}= round(abs(y(1,1)-T{i}(1,2))/dy+1);
n_T{i}= round((y_T{i}-1)*n+x_T{i});
end

% Calculating shortest unsmoothed paths between templates and riserbase  
% by using Dijkstra's algorithm. Also, plotting unsmoothed paths 
G=R;
length_unsmoothed=0;
L_unsmoothed=cell(1,Tn);
UnsmoothedPaths=cell(1,Tn);
for i=1:Tn
    caption1 = sprintf('%d',n_T{i});
    ni_T = findnode(G,caption1);
    caption2 = sprintf('%d',n_RBC);
    ni_RBC = findnode(G,caption2);
    [P,L_unsmoothed{i}] = shortestpath(G,ni_T,ni_RBC); %available MATLAB function for the Dijkstra's algorithm
    UnsmoothedPaths{i}(:,1)=G.Nodes{P(1,:),1};
    UnsmoothedPaths{i}(:,2)=G.Nodes{P(1,:),2};
    UnsmoothedPaths{i}(:,3)=G.Nodes{P(1,:),3};
    figure(2)
    plot3(G.Nodes{P(1,:),1},G.Nodes{P(1,:),2},G.Nodes{P(1,:),3}, 'color', 'b', 'linewidth', 2)
    hold on
    plot3(G.Nodes{P(1,1),1},G.Nodes{P(1,1),2},G.Nodes{P(1,1),3},'b.', 'MarkerSize',  20)
    caption3 = sprintf('T%d', i);
    text(G.Nodes{P(1,1),1},G.Nodes{P(1,1),2},G.Nodes{P(1,1),3},caption3, 'fontsize', 18)
    length_unsmoothed=length_unsmoothed+L_unsmoothed{i};
    a=numel(P);
    P(a)=[];
    P(a-1)=[];
    G = rmnode(G,P(:));
end
caption2 = sprintf('%d',n_RBC);
ni_RBC = findnode(G,caption2);
plot3(G.Nodes{ni_RBC,1},G.Nodes{ni_RBC,2},G.Nodes{ni_RBC,3},'b.', 'MarkerSize',  20)
text(G.Nodes{ni_RBC,1},G.Nodes{ni_RBC,2},G.Nodes{ni_RBC,3},'RB', 'fontsize', 18)
zlim([-180 180])    %This figure settings have been made for the used data
xlim([-35000 35000]) %for other data these can be changed 
ylim([-35000 35000])
xlabel('x');
ylabel('y');
zlabel('z');
pbaspect([1 1 1]);
grid off
hold off

%Smoothing lines to prevent slipping of flowlines while installation
for i=1:Tn
    SmoothPaths{i}=pathsmoothing(UnsmoothedPaths{i},Rmin); 
                    %Written MATLAB function for smoothing process
end

%Plotting Smoothed Lines
for i=1:Tn
figure(3)
plot3(SmoothPaths{i}(:,1),SmoothPaths{i}(:,2),SmoothPaths{i}(:,3), 'color', 'b', 'linewidth', 2) ;
zlim([-180 180]) %This figure settings have been made for the used data
xlim([-35000 35000]) %for other data these can be changed 
ylim([-35000 35000])
hold on
plot3(SmoothPaths{i}(1,1),SmoothPaths{i}(1,2),SmoothPaths{i}(1,3),'b.', 'MarkerSize',  20)
caption4 = sprintf('T%d', i);
text(SmoothPaths{i}(1,1),SmoothPaths{i}(1,2),SmoothPaths{i}(1,3),caption4, 'fontsize', 18)
end
plot3(G.Nodes{ni_RBC,1},G.Nodes{ni_RBC,2},G.Nodes{ni_RBC,3},'b.', 'MarkerSize',  20)
text(G.Nodes{ni_RBC,1},G.Nodes{ni_RBC,2},G.Nodes{ni_RBC,3},'RB', 'fontsize', 18)
xlabel('x');
ylabel('y');
zlabel('z');
pbaspect([1 1 1]);
grid off
hold off

%Calculating total length of smoothed lines
length_smoothed=0;
L_smoothed=cell(1,Tn);
for i=1:Tn
    L_smoothed{i}=smoothedpathlength(SmoothPaths{i}(:,1),SmoothPaths{i}(:,2),SmoothPaths{i}(:,3)); 
                    %Written function for calculating length
    length_smoothed=length_smoothed+L_smoothed{i};
end
X = sprintf('Total length of the flowlines is equal to %d m', length_smoothed);
disp(X);

    
