function [total_length] = smoothedpathlength(x_points,y_points,z_points)
%This function calculates length of smoothed lines
Points = [x_points(:),y_points(:),z_points(:)];
num_dim = size(Points,2);
% calculating total length of euclidean distances
each_eudist = sqrt(sum(diff(Points,[],1).^2,2));
length_eudist = each_eudist;
% determining 3rd order polynomials for the sections
splines = cell(1,num_dim);
spline_d = splines;
for i = 1:num_dim
      splines{i} = spline([0;cumsum(length_eudist)],Points(:,i));
      num_coef = numel(splines{i}.coefs);
      if num_coef < 4
        splines{i}.coefs = [zeros(1,4-num_coef),splines{i}.coefs];
        splines{i}.order = 4;
      end
  % Differentiation
  differentiation_array = [3 0 0;0 2 0;0 0 1;0 0 0];
  x_p = splines{i};
  x_p.coefs = x_p.coefs*differentiation_array;
  x_p.order = 3;
  spline_d{i} = x_p;
end
% Numerical integration
polynomial_array = zeros(num_dim,3);
for i = 1:splines{1}.pieces
  % extract polynomials for the derivatives
  for j = 1:num_dim
    polynomial_array(j,:) = spline_d{j}.coefs(i,:);
  end
  each_eudist(i) = quadgk(@(t) integ_equation(t),0,length_eudist(i));
                 %Available MATLAB function for Gaussian-Kronrod quadrature
                 %method of numerical integration
end
total_length = sum(each_eudist);

%Equation for integration
  function [equation] = integ_equation(t)
    % sqrt((dx/dl)^2 + (dy/dl)^2 + (dz/dl)^2)
    equation = zeros(size(t));
    for k = 1:num_dim
      equation = equation + polyval(polynomial_array(k,:),t).^2;
    end
    [equation] = sqrt(equation);
  end 
end 

